<?php  
            require('/srv2/pegasus.isi.edu/includes/common.php'); 
            pegasus_header("10.9. Integrity Checking");
        ?><div class="breadcrumbs">
<span class="breadcrumb-link"><a href="index.php">Pegasus 4.9.3 User Guide</a></span> &gt; <span class="breadcrumb-link"><a href="data_management.php">Data Management</a></span> &gt; <span class="breadcrumb-node">Integrity Checking</span>
</div><hr><div class="section">
<div class="titlepage"><div><div><h2 class="title" style="clear: both">
<a name="integrity_checking"></a>10.9. Integrity Checking</h2></div></div></div>
<div class="toc"><dl class="toc">
<dt><span class="section"><a href="integrity_checking.php#integrity_checking_stats">10.9.1. Integrity Checking Statistics</a></span></dt>
<dt><span class="section"><a href="integrity_checking.php#integrity_checking_dials">10.9.2. Integrity Checking Dials</a></span></dt>
<dt><span class="section"><a href="integrity_checking.php#integrity_checking_rc">10.9.3. Specifying Checksums in Replica Catalog</a></span></dt>
</dl></div>
<p>Pegasus adds checksum computation and integrity checking steps for
    non shared filesystem deployments (nonsharedfs and condorio). The main
    motivation to do this is to ensure that any data transferred for a
    workflow does not get inadvertently corrupted during data transfers
    performed during workflow execution, or at rest at a staging site. Users
    now have options to specify <span class="bold"><strong>sha256</strong></span>
    checksums for the input files in the replica catalog. If checksums are not
    provided, then Pegasus will compute the checksums for the files during
    data transfers, and enforce these checksums whenever a PegasusLite job
    starts on a remote node. The checksums for outputs created by user
    executable are generated and published by <span class="emphasis"><em>pegasus-kickstart
    </em></span>in it's provenance record. The kickstart output is brought back
    to the submit host as part of the job standard output using in-built
    HTCondor file transfer mechanisms. The generated checksums are then
    populated in the Stampede workflow database.</p>
<p>PegasusLite wrapped jobs invoke
    <span class="emphasis"><em>pegasus-integrity-check</em></span> before launching any
    computational task. <span class="emphasis"><em>pegasus-integrity-check</em></span> computes
    checksums on files and compares them against existing checksum values
    passed to it in its input. We also have extended our transfer tool
    pegasus-transfer to invoke pegasus-integrity check after completing the
    transfer of files.</p>
<p>Integrity checks in the workflows are implemented at 3 levels</p>
<div class="orderedlist"><ol class="orderedlist" type="1">
<li class="listitem"><p>after the input data has been staged to staging server -
        pegasus-transfer verifies integrity of the staged files.</p></li>
<li class="listitem"><p>before a compute task starts on a remote compute node - This
        ensures that checksums of the data staged in match the checksums
        specified in the input replica catalog or the ones computed when that
        piece of data was generated as part of previous task in the
        workflow.</p></li>
<li class="listitem"><p>after the workflow output data has been transferred to storage
        servers - This ensures that output data staged to the final location
        was not corrupted in transit.</p></li>
</ol></div>
<p>The <a class="link" href="integrity_checking.php#fig_pegasus_integrity_checking" title="Figure 10.6. Pegasus Integrity Checking">figure</a>
    below illustrates the points at which integrity checks are implemented. In
    our approach, the reference checksums for the input files for a job are
    sent to the remote node where a job executes using in-built HTCondor file
    transfer mechanism.</p>
<div class="figure">
<a name="fig_pegasus_integrity_checking"></a><p class="title"><b>Figure 10.6. Pegasus Integrity Checking</b></p>
<div class="figure-contents"><div class="mediaobject" align="center"><img src="images/pegasus-integrity-checking.png" align="middle" height="450" alt="Pegasus Integrity Checking"></div></div>
</div>
<br class="figure-break"><p>Currently, there are few scenarios where integrity checks will not
    happen in case of non shared filesystem deployments</p>
<div class="itemizedlist"><ul class="itemizedlist" style="list-style-type: disc; ">
<li class="listitem"><p>checksums are not enforced for user executables specified in the
        transformation catalog. In future, we plan to support checksumming for
        staged executable.</p></li>
<li class="listitem"><p>If you have set pegasus.transfer.bypass.input.staging to true to
        enable the bypass of staging of input files via the staging server,
        and have not specified the checksums in the replica catalog.</p></li>
</ul></div>
<div class="section">
<div class="titlepage"><div><div><h3 class="title">
<a name="integrity_checking_stats"></a>10.9.1. Integrity Checking Statistics</h3></div></div></div>
<p><a class="link" href="cli-pegasus-statistics.php" title="pegasus-statistics">pegasus-statistics</a>
      now includes a section containing integrity statistics:</p>
<pre class="programlisting">
	# Integrity Metrics
	# Number of files for which checksums were compared/computed along with total time spent doing it. 
	171 files checksums generated with total duration of 8.705 secs
	
	# Integrity Errors
	# Total:    
	#       Total number of integrity errors encountered across all job executions(including retries) of a workflow. 
	# Failures: 
	#       Number of failed jobs where the last job instance had integrity errors.
	Failures: 0 job failures had integrity errors
</pre>
</div>
<div class="section">
<div class="titlepage"><div><div><h3 class="title">
<a name="integrity_checking_dials"></a>10.9.2. Integrity Checking Dials</h3></div></div></div>
<p>Currently we support following dials for integrity
      checking.</p>
<div class="itemizedlist"><ul class="itemizedlist" style="list-style-type: disc; ">
<li class="listitem"><p><span class="bold"><strong>none</strong></span> - no integrity
          checking</p></li>
<li class="listitem"><p><span class="bold"><strong>full </strong></span>- full integrity
          checking for non shared filesystem deployments at the 3 levels
          described in this section.</p></li>
</ul></div>
<p>By default integrity checking dial is set to full . To change this
      you can set the following property</p>
<pre class="programlisting">pegasus.integrity.checking    none|full</pre>
</div>
<div class="section">
<div class="titlepage"><div><div><h3 class="title">
<a name="integrity_checking_rc"></a>10.9.3. Specifying Checksums in Replica Catalog</h3></div></div></div>
<p>For raw input files for your workflow you can specify the
      checksums along with file locations in the Replica Catalog. Pegasus will
      check against these checksums when a PegasusLite job starts up on a
      remote node. If checksums are not specified, then Pegasus will compute
      them during the data transfer to the staging site, and use them.</p>
<p>To specify checksums in replica catalog, you need to specify two
      additonal attributes with your LFN -&gt; PFN mapping.</p>
<div class="orderedlist"><ol class="orderedlist" type="1">
<li class="listitem"><p><span class="bold"><strong>checksum.type</strong></span> The checksum
          type. Currently only type of sha256 is supported</p></li>
<li class="listitem"><p><span class="bold"><strong>checksum.value</strong></span> The checksum
          for the file</p></li>
</ol></div>
<p>For example here is how you would specify the checksum for a file
      in a file based replica catalog</p>
<pre class="programlisting"># file-based replica catalog: 2018-10-25T02:10:02.293-07:00
f.a file:///lfs1/input-data/f.a checksum.type="sha256" checksum.value="ca8ed5988cb4ca0b67c45fd80fd17423aba2a066ca8a63a4e1c6adab067a3e92" site="condorpool"
</pre>
</div>
</div><div class="navfooter">
<hr>
<table width="100%" summary="Navigation footer">
<tr>
<td width="40%" align="left">
<a accesskey="p" href="metadata.php">Prev</a> </td>
<td width="20%" align="center"><a accesskey="u" href="data_management.php">Up</a></td>
<td width="40%" align="right"> <a accesskey="n" href="optimization.php">Next</a>
</td>
</tr>
<tr>
<td width="40%" align="left" valign="top">10.8. Metadata </td>
<td width="20%" align="center"><a accesskey="h" href="index.php">Table of Contents</a></td>
<td width="40%" align="right" valign="top"> Chapter 11. Optimizing Workflows for Efficiency and Scalability</td>
</tr>
</table>
</div><?php  
            pegasus_footer();
        ?>
